.\" @(#) mgif.man 3.5 91/6/15 rosenkra
.\" by rosenkra@convex.com (Bill Rosenkranz)
.\"
.TH MGIF 1 LOCAL "Version 3.5" "15 June 1991"
.SH NAME
mgif \- display a GIF and Degas images in monochrome
.SH SYNOPSIS
.\"	mgif [ -i ] [ -v ] [ -N ] [ -pi1 | -pi2 ] file ...
.B mgif
\&[
.B \-i
\&] [
.B \-v
\&] [
.B \-N
\&] [
.B \-pi1
\&|
.B \-pi2
\&]
.IR file ...
.br
.\"	mgif -fl [ -m ] file ...
.B mgif
.B \-fl
\&[
.B \-m
\&]
.IR file ...
.SH DESCRIPTION
.B Mgif
displays color
.SM GIF
and Degas
.SM PI1
or
.SM PI2
images on a monochrome ST display.
.SM GIF
files carry the extensions \*(lq.gif\*(rq and are compressed using
.SM LZW
codes.
It uses Floyd-Steinberg dithering to simulate color.
In addition, this new version implements \*(lqflicker\*(rq palettes to
simulate grayscale on a monochrome screen.
All things considered, it does a decent job.
.PP
.B Mgif
now also contains numerous image processing functions to enhance images.
Included are:
.sp
.nf
	Histogram equalization
	Contrast stretching
	Blurring
	Brightening (or darkening)
	Convolutions (numerous)
	Enlarge/reduce
	Inversion (negation with threshold)
	Log scaling
	Thresholding
	Median filtering
	Histogram plot
.fi
.PP
.B Mgif
allows each image processing function to be cummulative or each
operation can be independent.
.B Mgif
is interactive (not a
.SM GEM
program) and prompts for commands.
It also has online help and can save the \*(lqflicker\*(rq screen
image to a special file (*.fl) for viewing.
.B Mgif
can also view these files.
.SH OPTIONS
The following command line options are recognized:
.\"
.\"
.IP "\fB\-i\fR"
Inquire about the file but do not display it.
Output contains information like image size, number of bits per pixel,
and whether or not the image is interlaced or sequential.
.\"
.\"
.IP "\fB\-v\fR"
Verbose output.
.\"
.\"
.IP "\fB\-N\fR"
Use the classic
.SM NTSC
equation for computing luminescense from
.SM RGB
values, that is:
.sp
.RS
lum = 0.299 R + 0.587 G + 0.114 B
.RE
.sp
Otherwise the values are unweighted.
This option only makes sense for files with color maps in which the
entry for each color uses different values for red, green, and blue.
It is not recommended to use this for existing grayscale images.
Note that the actual equation used is:
.sp
.RS
lum = (30 R + 59 G + 11 B) / 100
.RE
.sp
which is close enough.
Also note that
.SM NTSC
luminescense is a global option and must be set on program entry.
.\"
.\"
.IP "\fB\-pi1\fR"
Files specified are Degas
.SM PI1
files rather than
.SM GIF
files, i.e. low resolution 320x200 16 color images.
.\"
.\"
.IP "\fB\-pi2\fR"
Files specified are Degas
.SM PI2
files rather than
.SM GIF
files, i.e. medium resolution 640x200 4 color images.
You will have to either shrink the horizontal axis or expand the vertical
axis after each image is read since the aspect ratio is wrong.
Since some images react better to shrinking than expanding, the choice
is left to the user.
It helps to blur the image first, especially when shrinking, since there
are only 4 intensities to start with.
Blurring will increase the number of
intensities and shrinking will reduce the effect of the blur.
This is generally optimal for these images.
.\"
.\"
.IP "\fIfile\fR"
One or more
.SM GIF
files (or Degas files if
.B \-pi1
or
.B \-pi2
is specified).
.\"
.\"
.PP
.B Mgif
can also be used as a simple flicker file viewer.
To do so, you must specify the
.B \-fl
option.
Only the
.B \-m
option is recognized in this case:
.\"
.\"
.IP "\fB\-fl\fR"
Just display flicker files (*.fl).
With no other options, each file is displayed sequentially.
Any key entered goes to the next file if there are any or
quits.
.\"
.\"
.IP "\fB\-m\fR"
This optional
flag will attempt to create a mosaic of multiple files.
Either two 320x400, four 320x200, or six 200x200 images are displayed
at the same time.
If images are smaller or larger than these sizes, tile to a size that fits.
For example two 320x300 images will be tiled as if they are 320x400.
.IP "\fIfile\fR"
In this case,
.I file
is one or more .fl files.
.\"
.\"
.SH "INTERACTIVE COMMANDS"
The following commands are available at the prompt:
.\"
.\"
.SS "Control Commands"
.IP "\fB?\fR"
help
.sp
Clears the screen and prints a brief summary of commands.
.\"
.\"
.IP "\fBq\fR"
quit (next image)
.sp
Quit does not exit the program.
It just moves to the next image from the command line.
.\"
.\"
.IP "\fBESC\fR"
exit program
.sp
Cleans up and exits the program.
.\"
.\"
.IP "\fBSPACE\fR"
draw image
.sp
The current image, either original or new, is drawn (or redrawn).
.\"
.\"
.IP "\fBw\fR"
what
.sp
Display information about the image.
.\"
.\"
.IP "\fBhi\fR"
histogram
.sp
Plots the histogram of the original or new image, depending on which
mode is toggled.
The histogram is based on intensity only.
Older versions of the program displayed
.SM RGB
histograms as well.
That option was dropped, though it was sometimes useful.
Note that if
.B o
is toggled for original image, you should toggle
.B o
again to plot a histogram of a transformed image, then toggle back
to work on the original.
.\"
.\"
.IP "\fBo\fR"
original/new image
.sp
This toggles two sorts of behavior: 1) all transformations take place
on the
.I original
image and are not cummulative, 2) all transformations
take place on the
.I new
(last) image and are cummulative.
.\"
.\"
.IP "\fBf\fR"
file operations
.sp
Currently, the only file operation is to save the last screen image
to a special file having a \*(lq.fl\*(rq suffix, a so called \*(lqflicker\*(rq
file.
This file contains the three screen images which are used by the flicker
routine to simulate grayscale.
These are the actual screen bitmaps, truncated to the actual image size.
Note that in order to save an image in this format, you
.I must
first display it completely by the flicker routine.
You should not escape the actual screen draw but let it finish.
.\"
.\"
.IP "\fBcp\fR"
image copy
.sp
You can copy the new, transformed image to the original and visa versa.
The former is the most useful.
.\"
.\"
.SS "Display Options"
.IP "\fBn\fR"
noise
.sp
Adds noise not to the image but to the \*(lqflicker\*(rq process itself.
Can help in images which have artificial banding as a result of the
dithering process.
You are asked for a value,
which if 0, turns off noise addition.
Otherwise, it is a percentage (0 to 100).
Each time
.B o
is toggled, noise is reset to 0 (i.e. off).
.\"
.\"
.IP "\fBg\fR"
Laplace filter
.sp
This, like noise, is not applied to the image itself but rather in
the flicker process.
It helps to define hard edges in images and implements a Laplace filter.
A quantity,
.BR Beta ,
is required.
Use integers (1, 2, etc.) for this value.
A value of 1 corresponds to Beta of 0.25,
a value of 2 corresponds to Beta of 0.50, etc.
A value of 0 turns off Laplace filtering.
Each time
.B o
is toggled, Beta is reset to 0 (i.e. Laplace filtering off).
.\"
.\"
.SS "Frame Processes"
.IP "\fBe\fR"
enlarge
.sp
Enlarges images by 2x (only).
You can choose the dimension to enlarge as either horizontal, vertical,
or both.
Note that if the size of an image is such that enlarging it will exceed
the amount of memory allocated for the image, the enlarge is aborted.
The enlarged image can be larger than the screen, but must fit in the raster
memory.
Enlarging is done by shifing pixels and filling in the holes within a scan
line with the average of adjacent pixels, and between scan lines by
averaging adjacent scan lines to create the new one.
.\"
.\"
.IP "\fBs\fR"
shrink
.sp
Reduces images by 2x (only).
Makes them half size in height and width.
You can choose the dimension to reduce as either horizontal, vertical,
or both.
Reduction is a simple decimation which throws away every other point in
a scan line and every other scan line.
.\"
.\"
.IP "\fBz\fR"
zoom
.sp
The image is drawn (without flicker) then gridded with a 20x20 grid,
then a box appears
representing the zoom window which is always half the height and width
of the image.
Move the box with the arrow keys.
By themselves, the arrow keys move 8 pixels in the corresponding direction.
Shifted arrows move the box 1 pixel at a time.
Detect the final location with the
.SM INSERT
key.
Any other key cancels.
Once the zoom rectangle is defined, the image defined is expanded to fit
the current image size (similar to enlarge with both directions specified).
.\"
.\"
.IP "\fBcu\fR"
cut
.sp
Cuts out a rectangular area of the image for scrutiny.
You can choose to cut areas which are partially or fully out of the
display area.
First, the image is redisplayed without flicker.
Then a 20x20 pixel grid is overlaid.
A crosshair system is then drawn.
Move the crosshairs with the arrow keys (see
.SM zoom
section).
Detect the upper left corner first with the
.SM INSERT
key.
Then do the same for the lower right corner.
The cut image is drawn with the same size as the cut area which can then
be enlarged if desired.
Use zoom to cut and enlarge (with restrictions) all at once.
Cutting is helpful for a number of uses including: 1) zooming in on a smaller
portion, 2) to trim artificial borders around an image, 3) for artistic
cropping.
.\"
.\"
.IP "\fBr\fR"
rotate
.sp
You can only rotate the original image in this version.
If you are in \*(lqnew\*(rq mode, you are asked if you want to proceed
(because any transformations done in the new buffer will be lost) and
if you answer \*(lqyes\*(rq, you are placed in orig mode where the
transformation takes place.
You are asked for an angle which must be either -90 (for 90 degree clockwise
rotation), 90 (for 90 degree counter clockwise rotation) or 180.
After the image is rotated, you are placed in \*(lqnew\*(rq mode where
all suggessive transformations accumulate.
The original image, however, is also rotated.
.\"
.\"
.IP "\fBmi\fR"
mirror
.sp
You can mirror about a horizontal axis (top to bottom swap) or a vertical
axis (left to right swap).
.\"
.\"
.SS "Point Processes"
.IP "\fBhe\fR"
histogram equalization
.sp
Histogram equalization attempts to smooth out images with skewed or narrow
histograms.
A histogram is a graph showing the relative number of pixels with each
possible intensity value.
The practical use is to make images more uniform in contrast by
making the histogram more uniform and hence representing all intensities
as equally as possible.
It is often not possible to achive a very smooth histogram, but more often
than not it is possible to improve image quality.
Histogram equalization can also help to bring out structure hidden in shadow.
The technique used here does not randomly redistribute pixels to new
intensity levels.
It just changes the intensity of all pixels with a certain intensity,
thus stretching the histogram so that each intensity is represented
uniformly.
.\"
.\"
.IP "\fBi\fR"
inverse (negation)
.sp
Invert an image, i.e. make low intensity pixels bright and visa versa.
Inversion is implemented here more generally.
A threshold is needed.
If it is 0, the entire image is inverted.
If it is a negative value between 0 and -255, only pixels
.I below
that level are inverted.
If it is a positive value between 0 and 255, only pixels
.I above
that level are inverted.
.sp
The basic algorithm is:
.sp
.nf
	For threshold = 0:

	    new[i] = 255 - old[i]

	For threshold < 0:

	    new[i] = 255 - old[i], old[i] < -threshold
	           = old[i], otherwise

	For threshold > 0:

	    new[i] = 255 - old[i], old[i] > threshold
	           = old[i], otherwise
.fi
.\"
.\"
.IP "\fBl\fR"
log scaling
.sp
Application of log scaling helps some images.
Here each pixel's value is scaled to the ratio of the logarithm of the
pixel's intensity and the maximum intensity in the image.
.sp
The basic algorithm is:
.sp
.nf
	new[i] = maxval * log(old[i]) / log(maxval)

	maxval is highest intensity in the image
.fi
.\"
.\"
.IP "\fBt\fR"
threshold (black & white)
.sp
This process makes all pixels above a given threshold intensity
white and all below the threshold black.
It is useful in producing line art.
.sp
The basic algorithm is:
.sp
.nf
	new[i] = 255, if old[i] > threshold
	       =   0, if old[i] <= threshold
.fi
.\"
.\"
.IP "\fBco\fR"
contrast expansion
.sp
Attempts to expand the contrast of an image to cover the entire range
of 256 allowable grayscales.
Somewhat similar to histogram equalization but it does not redistribute
actual pixels.
You are asked for a threshold which is a pixel count.
This determines the lowest and highest intensities, based on the
histogram, which define the range to be stretched.
What happens is that the histogram is searched beginning at the low end
for the first intensity level with more pixels than the input threshold.
This is also done starting at the high end.
A range is thus defined.
Entering 0 will define a range starting with the lowest (and highest)
intensity for which there are more than one points.
.sp
The basic algorithm is:
.sp
.nf
	new[i] = 255 * (old[i] - lo) / (hi - lo)

	where lo to hi is new contrast range
.fi
.sp
The contrast range is defined by the intensities of bins which
exceed the given threshold pixel count at both low and high ends of
the histogram.
.\"
.\"
.IP "\fBbr\fR"
brighten (or darken)
.sp
Here a constant intensity is added (or subtracted) from every pixel.
A negative number causes image darkening and a positive number causes
image brightening.
Pixels which would otherwise go less than 0 or greater than 255 are limited
to these values, respectively.
.sp
The basic algorithm is:
.sp
.nf
	new[i] = old[i] + constant
.fi
.\"
.\"
.SS "Area Processes"
.IP "\fBbl\fR"
blur
.sp
Blurring an image can also help contrast, though at the expense of loss
of detail.
Here a 3x3 neighborhood is used.
Often in practice a 5x5 neighborhood is used, but that tends to take a
relatively long time.
.sp
The basic algorithm is (j runs through the 3x3 neighborhood):
.sp
.nf
	new[i] = sum (old[i] ) / 9
	            j       j
.fi
.\"
.\"
.IP "\fBme\fR"
median filter
.sp
Median filtering can remove noise from an image.
Here a 3x3 neighborhood is used with the median value of each of the 9
pixels replacing the pixel of interest.
This is done in place so that the next pixel examined within the row or the
next row will see pixels already filtered.
.sp
The basic algorithm is (j runs through the 3x3 neighborhood):
.sp
.nf
	new[i] = median (old[i] )
	               j       j
.fi
.\"
.\"
.IP "\fBv\fR"
convolution filters 
.sp
Apply one of a number of 3x3 convolutions to an image.
Choose one of the built\-in filters (number greater than 0) or enter 0
for user defined.
In the latter case, you enter the 9 elements of the convolution kernel along
with a flag if there will be a multiplier (or divisor).
Then enter that scaling factor.
All quantities are to be integers.
.sp
The basic algorithm is (j runs through the 3x3 neighborhood):
.sp
.nf
	new[i] = sum (old[i]  * k )
		    j       j    j
or

	new[i] = sum (old[i]  * k ) * multiplier
		    j       j    j
or

	new[i] = sum (old[i]  * k ) / divisor
		    j       j    j

	where k  is the corresponding kernel value.
	       j
.fi
.sp
It is not a matrix multiply, rather an element by element
multiply and summation.
.SH FILES
.nf
*.gif   GIF image files
*.pi1   Degas low resolution files
*.pi2   Degas medium resolution files
*.fl    flicker files
.fi
.SH "SEE ALSO"
gif(5), showfl(1L), pi1(1L)
.br
Gonzalez and Wintz, \fIDigital Image Processing\fR.
.br
Lindley, \fIPractical Image Processing In C\fR.
.br
Burger and Gillies, \fIInteractive Computer Graphics\fR.
.SH DIAGNOSTICS
Various warnings attempting to isolate corrupted files.
The usual messages about errors openning files, etc.
If an error occurs during a processing function, the prompt line
will indicate the error.
.SH NOTES
Source code has standalone versions (i.e. single entry point) for
both reading
.SM GIF
files and displaying grayscale bitmaps with flicker.
.SH BUGS
As compiled,
.B mgif
handles images up to 400,000 pixels or about 640x620 (or any other
combination totaling less than 400,000).
It needs dynamic memory but I generally stay away from
.SM GEMDOS
.IR Malloc (2),
prefering libc
.IR malloc (3)
(i.e. off stack in my library).
.PP
.SM GIF
decoding is slow and could be faster.
Some of the area processes (blur, etc.) could also be faster.
.PP
Log scaling needs logarithms but I did not want to use floating point
so I get a crude (two significant digit) base 2 log.
.PP
The flicker file (*.fl) format is not fully defined.
.SH TODO
Though
.B mgif
is now fairly complete, there are some things that would be nice to add.
Here is the short list:
.sp
.nf
	\(bu Merge images

	\(bu Batch processing

	\(bu Save history of changes for batch

	\(bu Histograms by pixel range directly

	\(bu Comparative flicker (i.e. A/B)
.fi
.SH AUTHOR
Bill Rosenkranz (rosenkra@convex.com)
.br
Flicker adopted from code by Klaus Pedersen (micro@imada.dk)
.SH VERSION
mgif 3.5 91/6/15 rosenkra
